@@ -13,7 +13,7 @@ package Tk::CmdLine; # -*-Perl-*-
 #/----------------------------------------------------------------------------//
 
 use vars qw($VERSION);
-$VERSION = '3.018'; # $Id: //depot/Tk8/Tk/CmdLine.pm#18$
+$VERSION = '3.027'; # $Id: //depot/Tk8/Tk/CmdLine.pm#27 $
 
 use 5.004;
 
@@ -24,10 +24,6 @@ use base qw(AutoLoader);
 
 sub import {  } # else we inherit AutoLoader's import
 
-use vars qw($ThisModule);
-
-$ThisModule = ref bless []; # the name of this package
-
 use vars qw($OBJECT); # define the current object
 $OBJECT = undef; # global so that it will be accessible in AutoLoader methods
 
@@ -55,9 +51,7 @@ sub new # Tk::CmdLine::new()
         motif       => $Tk::strictMotif,
         resources   => {} };
 
-    bless($self, $class);
-
-    return $self;
+    return bless $self,$class;
 }
 
 #/----------------------------------------------------------------------------//
@@ -143,15 +137,11 @@ my %Method = (
     xrm          => 'Resource_'
 );
 
-sub process { SetArguments(); } # for compatibility with old code
 
 sub SetArguments # Tk::CmdLine::SetArguments([@argument])
 {
-    my $self = (@_ # define the object as necessary
-        ? ((ref($_[0]) eq $ThisModule)
-            ? shift(@_)
-            : (($_[0] eq $ThisModule) ? shift(@_) : 1) && ($OBJECT ||= $ThisModule->new()))
-        : ($OBJECT ||= $ThisModule->new()));
+    my $self = (@_) ? shift : __PACKAGE__;
+    
     $OBJECT = $self; # update the current object
     $self->{argv}   = (@_ ? [ @_ ] : \@ARGV);
     $self->{offset} = 0; # its existence will denote that this method has been called
@@ -194,6 +184,8 @@ sub SetArguments # Tk::CmdLine::SetArguments([@argument])
     return $self;
 }
 
+*process = \&SetArguments; # for compatibility with old code
+
 #/----------------------------------------------------------------------------//
 #/ Get the value of a configuration option (default: -class).
 #/   Returns the option value.
@@ -201,11 +193,11 @@ sub SetArguments # Tk::CmdLine::SetArguments([@argument])
 
 sub cget # Tk::CmdLine::cget([$option])
 {
-    my $self = (@_ # define the object as necessary
-        ? ((ref($_[0]) eq $ThisModule)
-            ? shift(@_)
-            : (($_[0] eq $ThisModule) ? shift(@_) : 1) && ($OBJECT ||= $ThisModule->new()))
-        : ($OBJECT ||= $ThisModule->new()));
+    my $self = (@_) ? shift : __PACKAGE__;
+    unless (ref $self)
+     {
+      $self = $OBJECT || $self->new;
+     }
     $OBJECT = $self; # update the current object
     my $option = shift(@_) || '-class';
 
@@ -218,15 +210,13 @@ sub cget # Tk::CmdLine::cget([$option])
 
 sub CreateArgs # Tk::CmdLine::CreateArgs()
 {
-    my $self = (@_ # define the object as necessary
-        ? ((ref($_[0]) eq $ThisModule)
-            ? shift(@_)
-            : (($_[0] eq $ThisModule) ? shift(@_) : 1) && ($OBJECT ||= $ThisModule->new()))
-        : ($OBJECT ||= $ThisModule->new()));
+    my $self = (@_) ? shift : __PACKAGE__;
+    unless (ref $self)
+     {
+      $self = $OBJECT || $self->new;
+     }
     $OBJECT = $self; # update the current object
-
     $self->SetArguments() unless exists($self->{offset}); # set arguments if not yet done
-
     return $self->{config};
 }
 
@@ -236,7 +226,7 @@ sub Tk::MainWindow::apply_command_line
 {
     my $mw = shift(@_);
 
-    my $self = ($OBJECT ||= $ThisModule->new());
+    my $self = ($OBJECT ||= __PACKAGE__->new());
 
     $self->SetArguments() unless exists($self->{offset}); # set arguments if not yet done
 
@@ -279,21 +269,22 @@ sub Tk::MainWindow::apply_command_line
         $self->{iconic} = 0;
     }
 
-	$Tk::strictMotif = $self->{motif};
+    $Tk::strictMotif = $self->{motif};
 
     # Both these are needed to reliably save state
     # but 'hostname' is tricky to do portably.
     # $mw->client(hostname());
-    # $mw->protocol('WM_SAVE_YOURSELF' => ['WMSaveYourself',$mw]);
-
+    $mw->protocol('WM_SAVE_YOURSELF' => ['WMSaveYourself',$mw]);
     $mw->command([ $self->{name}, @{$self->{command}} ]);
 }
 
 #/----------------------------------------------------------------------------//
 
+END { undef $OBJECT }
+
 1;
 
-__END__
+# __END__
 
 #/----------------------------------------------------------------------------//
 #/ Set the initial resources.
@@ -302,11 +293,11 @@ __END__
 
 sub SetResources # Tk::CmdLine::SetResources((\@resource | $resource) [, $priority])
 {
-    my $self = (@_ # define the object as necessary
-        ? ((ref($_[0]) eq $ThisModule)
-            ? shift(@_)
-            : (($_[0] eq $ThisModule) ? shift(@_) : 1) && ($OBJECT ||= $ThisModule->new()))
-        : ($OBJECT ||= $ThisModule->new()));
+    my $self = (@_) ? shift : __PACKAGE__;
+    unless (ref $self)
+     {
+      $self = $OBJECT || $self->new;
+     }
     $OBJECT = $self; # update the current object
 
     $self->SetArguments() unless exists($self->{offset}); # set arguments if not yet done
@@ -342,12 +333,12 @@ sub SetResources # Tk::CmdLine::SetResources((\@resource | $resource) [, $priori
 
 sub LoadResources # Tk::CmdLine::LoadResources([%options])
 {
-
-    my $self = (@_ # define the object as necessary
-        ? ((ref($_[0]) eq $ThisModule)
-            ? shift(@_)
-            : (($_[0] eq $ThisModule) ? shift(@_) : 1) && ($OBJECT ||= $ThisModule->new()))
-        : ($OBJECT ||= $ThisModule->new()));
+    my $self = (@_) ? shift : __PACKAGE__;
+    unless (ref $self)
+     {
+      $self = $OBJECT || $self->new;
+     }
+	
     $OBJECT = $self; # update the current object
 
     $self->SetArguments() unless exists($self->{offset}); # set arguments if not yet done
@@ -401,8 +392,7 @@ sub LoadResources # Tk::CmdLine::LoadResources([%options])
                 };
             }
 
-            my @postfix = map({ $_ . '/' . $self->{config}->{-class} }
-                ('/' . $self->{translation}->{'%L'}), '');
+            my @postfix = map { $_ . '/' . $self->{config}->{-class} } ('/' . $self->{translation}->{'%L'}), '';
 
             ITEM: foreach $fileSpec (split(':', $xpath))
             {
@@ -482,9 +472,12 @@ sub LoadResources # Tk::CmdLine::LoadResources([%options])
     }
 
     return $self;
-}
+}    
 
-#/----------------------------------------------------------------------------//
+sub DESTORY
+{
+ # avoid AUTOLOAD
+}
 
 1;
 
