@@ -5,7 +5,7 @@ package Tk::Table;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '3.012'; # $Id: //depot/Tk8/Tk/Table.pm#12$
+$VERSION = '3.018'; # $Id: //depot/Tk8/Tk/Table.pm#18 $
 
 use Tk::Pretty;
 use AutoLoader;
@@ -124,7 +124,7 @@ sub constrain
  $n = $fixed if ($n < $fixed);
  for ($i= 0; $i < $fixed; $i++)
   {
-   $total += $a->[$i];
+    (defined($a->[$i])) && ($total += $a->[$i]);
   }
  for ($i=$n; $total < $pixels && $i < @$a; $i++)
   {
@@ -331,7 +331,7 @@ sub Layout
 sub QueueLayout
 {
  my ($m,$why) = @_;
- $m->DoWhenIdle(['Layout',$m]) unless ($m->{LayoutPending});
+ $m->afterIdle(['Layout',$m]) unless ($m->{LayoutPending});
  $m->{LayoutPending} |= $why;
 }
 
