@@ -22,18 +22,18 @@
 #ifndef NO_STDLIB_H
 #include <stdlib.h>
 #endif
-
+                
 #if !defined(__GNUC__) && !defined(_AIX)
 #ifdef __STDC__
 #ifndef STRINGIFY
-#define STRINGIFY(x)	    STRINGIFY1(x)
-#define STRINGIFY1(x)	    #x
-#endif
+#define STRINGIFY(x)        STRINGIFY1(x)
+#define STRINGIFY1(x)       #x
+#endif	/* STRINGIFY */
 #define __FUNCTION__ __FILE__ ":" STRINGIFY(__LINE__)
-#else /* STDC */
-#endif
+#else 	/* STDC */
 #define __FUNCTION__ ""
-#endif
+#endif	/* STDC */
+#endif	/* GNUC or AIX */
 
 #define USE_TCLALLOC 1
 
@@ -233,11 +233,7 @@ typedef long LONG;
  * Note: Tcl_ObjCmdProc procedures do not directly set result and freeProc.
  * Instead, they set a Tcl_Obj member in the "real" structure that can be
  * accessed with Tcl_GetObjResult() and Tcl_SetObjResult().
- */
-
-#ifndef LangCallback
-typedef struct LangCallback *LangCallback;
-#endif
+ */         
 
 #ifndef Tcl_Interp
 typedef struct Tcl_Interp
@@ -895,11 +891,20 @@ typedef struct Tcl_HashSearch {
  * event.
  */
 
+#if defined(USE_TCL_STRUCTS) || defined(TCL_EVENT_IMPLEMENT) 
+
 struct Tcl_Event {
     Tcl_EventProc *proc;	/* Procedure to call to service this event. */
     struct Tcl_Event *nextPtr;	/* Next in list of pending events, or NULL. */
 };
 
+#else 
+struct Tcl_Event {
+    void *(*spaceforproc)(void);
+    void *spaceforlink;
+};
+#endif
+
 /*
  * Positions to pass to Tcl_QueueEvent:
  */
@@ -1402,6 +1407,9 @@ EXTERN void		Tcl_PrintDouble _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN int		Tcl_PutEnv _ANSI_ARGS_((CONST char *string));
 EXTERN void		Tcl_QueueEvent _ANSI_ARGS_((Tcl_Event *evPtr,
 			    Tcl_QueuePosition position));
+EXTERN void		Tcl_QueueProcEvent _ANSI_ARGS_((Tcl_EventProc *proc,
+			    Tcl_Event *evPtr,
+			    Tcl_QueuePosition position));
 EXTERN int		Tcl_Read _ANSI_ARGS_((Tcl_Channel chan,
 	        	    char *bufPtr, int toRead));
 EXTERN void		Tcl_ReapDetachedProcs _ANSI_ARGS_((void));
@@ -1539,11 +1547,7 @@ EXTERN Tcl_Pid		Tcl_WaitPid _ANSI_ARGS_((Tcl_Pid pid, int *statPtr,
 			    int options));
 EXTERN int		Tcl_Write _ANSI_ARGS_((Tcl_Channel chan,
         		    char *s, int slen));
-
-typedef void (LangFreeProc) _ANSI_ARGS_((int count,Arg *blockPtr));
-
 EXTERN void		Tcl_AppendArg _ANSI_ARGS_((Tcl_Interp *interp, Arg));
-EXTERN void		Tcl_ArgResult _ANSI_ARGS_((Tcl_Interp *interp, Arg));
 EXTERN void		Tcl_IntResults _ANSI_ARGS_((Tcl_Interp *interp,int,int,...));
 EXTERN void		Tcl_DoubleResults _ANSI_ARGS_((Tcl_Interp *interp,int,int,...));
 
@@ -1557,12 +1561,6 @@ EXTERN int		Tcl_AfterObjCmd _ANSI_ARGS_((ClientData clientData,
 
 #define Tcl_GlobalEval(interp,cmd) LangEval(interp,cmd,1)
 EXTERN char *		LangMergeString _ANSI_ARGS_((int argc, Arg *args));
-EXTERN int		Lang_SplitList _ANSI_ARGS_((Tcl_Interp *interp,
-			    Arg list, int *argcPtr, Arg **argsPtr, 
-			    LangFreeProc **));
-EXTERN int		Lang_SplitString _ANSI_ARGS_((Tcl_Interp *interp,
-			    const char *list, int *argcPtr, Arg **argsPtr, 
-			    LangFreeProc **));
 
 EXTERN int LangEval _ANSI_ARGS_((Tcl_Interp *interp, char *cmd, int global));
 
@@ -1572,10 +1570,12 @@ EXTERN void LangSetDefault _ANSI_ARGS_((Arg *,char *));
 EXTERN void LangSetInt _ANSI_ARGS_((Arg *,int));
 EXTERN void LangSetDouble _ANSI_ARGS_((Arg *,double));
 EXTERN void LangSetArg _ANSI_ARGS_((Arg *,Arg));
-EXTERN Arg  LangStringArg _ANSI_ARGS_((char *));
 EXTERN int  LangCmpArg  _ANSI_ARGS_((Arg,Arg));
 EXTERN int  LangCmpOpt  _ANSI_ARGS_((char *opt,char *arg,size_t length));
 
+#define LangStringArg(s) Tcl_NewStringObj(s,-1)
+#define Tcl_ArgResult(interp,obj) Tcl_SetObjResult(interp,obj)
+
 /* FIXME:
    Tk will set freeProc as for Tcl e.g. NULL for statics & UIDs
    and to "free" for Tcl_Merge etc.
@@ -1592,19 +1592,11 @@ EXTERN Arg  LangCopyArg _ANSI_ARGS_((Arg));
 EXTERN void LangRestoreResult _ANSI_ARGS_((Tcl_Interp **,LangResultSave *));
 EXTERN LangResultSave *LangSaveResult _ANSI_ARGS_((Tcl_Interp **));
 
-EXTERN Arg *LangAllocVec _ANSI_ARGS_((int count));
-EXTERN void LangFreeVec _ANSI_ARGS_((int,Arg *));
-
 EXTERN void Tcl_SprintfResult _ANSI_ARGS_((Tcl_Interp *,char *,...));
 EXTERN char *Tcl_GetResult _ANSI_ARGS_((Tcl_Interp *));
 EXTERN void Tcl_Panic _ANSI_ARGS_((char *,...));
 #define panic Tcl_Panic
 
-EXTERN LangCallback *LangMakeCallback _ANSI_ARGS_((Arg));
-EXTERN Arg LangCallbackArg _ANSI_ARGS_((LangCallback *));
-EXTERN void LangFreeCallback _ANSI_ARGS_((LangCallback *));
-EXTERN int LangDoCallback _ANSI_ARGS_((Tcl_Interp *,LangCallback *,int result,int argc,...));
-EXTERN int LangMethodCall _ANSI_ARGS_((Tcl_Interp *,Arg,char *,int result,int argc,...));
 
 EXTERN int  LangNull _ANSI_ARGS_((Arg));
 
@@ -1640,10 +1632,6 @@ EXTERN Arg LangVarArg _ANSI_ARGS_((Var));
 EXTERN Arg Tcl_ResultArg _ANSI_ARGS_((Tcl_Interp *interp));
 EXTERN Arg LangScalarResult _ANSI_ARGS_((Tcl_Interp *interp));
 
-EXTERN int LangCmpCallback _ANSI_ARGS_((LangCallback *a,Arg b));
-
-EXTERN LangCallback *LangCopyCallback _ANSI_ARGS_((LangCallback *));
-
 EXTERN void		Tcl_WrongNumArgs _ANSI_ARGS_((Tcl_Interp *interp,
 			    int objc, Tcl_Obj *CONST objv[], char *message));
 
@@ -1667,6 +1655,13 @@ EXTERN Tcl_Command	Lang_CreateObject _ANSI_ARGS_((Tcl_Interp *interp,
 EXTERN int Lang_CallWithArgs _ANSI_ARGS_ ((Tcl_Interp *interp, 
 					char *sub, int argc, Arg *argv));
 
+#ifndef LangCallback
+typedef struct LangCallback *LangCallback;
+#endif
+
+EXTERN int LangDoCallback _ANSI_ARGS_((Tcl_Interp *,LangCallback *,int result,int argc,...));
+EXTERN int LangMethodCall _ANSI_ARGS_((Tcl_Interp *,Arg,char *,int result,int argc,...));
+
 EXTERN char *LangLibraryDir _ANSI_ARGS_((void));
 EXTERN void Lang_SetBinaryResult _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *string, int len, Tcl_FreeProc *freeProc));
@@ -1687,6 +1682,11 @@ EXTERN long Lang_OSHandle _ANSI_ARGS_((int fd));
 #pragma warning(disable:4101 4102 4244 4018)
 #pragma warning(disable:4133) /* init incompatible for xlib */
 #endif
+#endif   
+
+#if !defined(TCL_EVENT_IMPLEMENT)
+#include "tkEvent.h"          
+#include "tkEvent.m"          
 #endif
 
 #endif /* RESOURCE_INCLUDED */
